package nl.sbdeveloper.mctpaudio.listener;

import com.sk89q.worldguard.protection.regions.ProtectedRegion;
import nl.sbdeveloper.mctpaudio.MCTPAudio;
import nl.sbdeveloper.mctpaudio.managers.PinManager;
import nl.sbdeveloper.mctpaudio.managers.WGManager;
import net.raidstone.wgevents.events.RegionsEnteredEvent;
import org.bukkit.Location;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerMoveEvent;
import org.bukkit.event.player.PlayerTeleportEvent;
import org.json.simple.JSONObject;

import java.util.*;
import java.util.stream.Collectors;

/* Copyright (C) McThemeParks - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written by Stijn Bannink <stijnbannink23@gmail.com>, July 2020
 */

public class WGListener implements Listener {
    /**
     * Music detection
     */
    @EventHandler
    public void onMove(PlayerMoveEvent e) {
        handlePlayerMovement(e.getPlayer(), e.getFrom(), e.getTo());
    }

    @EventHandler
    public void onTeleport(PlayerTeleportEvent e) {
        handlePlayerMovement(e.getPlayer(), e.getFrom(), e.getTo());
    }

    private void handlePlayerMovement(Player player, Location from, Location to) {
        if (to == null || from.getWorld() == null || to.getWorld() == null) return;

        if (from.getBlockX() != to.getBlockX() || from.getBlockY() != to.getBlockY() || from.getBlockZ() != to.getBlockZ()) {
            Set<String> list = MCTPAudio.getPlugin().getConfig().getConfigurationSection("Regions").getKeys(false);

            if (!PinManager.hasPin(Objects.requireNonNull(player).getUniqueId())) return;

            List<String> fromRegions = WGManager.getRegionsIn(from).stream().map(ProtectedRegion::getId).collect(Collectors.toList());
            List<String> toRegions = WGManager.getRegionsIn(to).stream().map(ProtectedRegion::getId).collect(Collectors.toList());

            if ((Collections.disjoint(list, fromRegions) && !Collections.disjoint(list, toRegions)) || (!Collections.disjoint(list, fromRegions) && !Collections.disjoint(list, toRegions))) {
                //Walked in a region

                if (!Collections.disjoint(list, fromRegions) && !Collections.disjoint(list, toRegions)) {
                    Optional<String> name = fromRegions.stream().filter(list::contains).findFirst();
                    Optional<String> name2 = toRegions.stream().filter(list::contains).findFirst();

                    if (name.isPresent() && name2.isPresent() && name.get().equals(name2.get()))
                        return; //Beide heeft een region, en dat is dezelfde.

                    if (name.isPresent() && name2.isPresent()) {
                        String regionURL = MCTPAudio.getPlugin().getConfig().getString("Regions." + name.get());
                        String regionURL2 = MCTPAudio.getPlugin().getConfig().getString("Regions." + name2.get());

                        if (regionURL.equals(regionURL2))
                            return; //Beide heeft een region, niet dezelfde, maar wel met dezelfde muziek.
                    }
                }

                Optional<String> name = toRegions.stream().filter(list::contains).findFirst();
                if (!name.isPresent()) return;
                String regionURL = MCTPAudio.getPlugin().getConfig().getString("Regions." + name.get());

                JSONObject data = new JSONObject();
                data.put("task", "MUSIC");
                data.put("path", regionURL);
                data.put("uuid", player.getUniqueId().toString().replace("-", ""));
                MCTPAudio.getClient().sendData(data);
            } else if (!Collections.disjoint(list, fromRegions) && Collections.disjoint(list, toRegions)) {
                //Not in a region, stop...
                JSONObject data = new JSONObject();
                data.put("task", "MUSIC");
                data.put("path", "");
                data.put("uuid", player.getUniqueId().toString().replace("-", ""));
                MCTPAudio.getClient().sendData(data);
            }
        }
    }

    /**
     * Hue detection
     */
    @EventHandler
    public void onRegionEnter(RegionsEnteredEvent e) {
        if (e.getPlayer() == null) return;

        Set<String> list2 = MCTPAudio.getPlugin().getConfig().getConfigurationSection("HueRegions").getKeys(false);

        if (!Collections.disjoint(list2, e.getRegionsNames())) {
            //One element is the same -> In a region

            Optional<String> name = e.getRegionsNames().stream().filter(list2::contains).findFirst();
            if (!name.isPresent()) return;
            String configData = MCTPAudio.getPlugin().getConfig().getString("HueRegions." + name.get());

            if (configData == null) return;
            String[] configDataSplit = configData.split("_");

            int r = Integer.parseInt(configDataSplit[0]);
            int g = Integer.parseInt(configDataSplit[1]);
            int b = Integer.parseInt(configDataSplit[2]);
            String type = configDataSplit[3];
            Integer brightness = null;
            if (configDataSplit.length == 5) brightness = Integer.parseInt(configDataSplit[4]);

            if (!PinManager.hasPin(e.getPlayer().getUniqueId())) return;

            JSONObject data = new JSONObject();
            data.put("task", "HUE");
            data.put("rgb", r + ":" + g + ":" + b);
            data.put("type", type);
            if (brightness != null) data.put("brightness", brightness);
            data.put("uuid", e.getPlayer().getUniqueId().toString().replace("-", ""));
            MCTPAudio.getClient().sendData(data);
        }
    }
}
